th {
  font-size: 100%;
}

th.rotate {
  height: 225px;
  white-space: nowrap;
}

th.rotate > div {
  transform: 
    translate(0px, 100px)
    rotate(270deg);
  width: 20px;
}

.ui-tooltip {
  white-space: pre-wrap;
}

.ui-tooltip-content {
  font-size: 70%;
  word-wrap: break-word;
}

.done {
  background-color:#4fef4f;
}

.todo {
  background-color:#ef4f4f;
}

.green {
  background-color:#4fef4f;
}

.yellow {
    background-color:#ffff00;
}

.red {
  background-color:#ef4f4f;
}

.step {
  background-color:#e0e0e0;
}
