div.sitecheck {
    clear:right;
    float:left;
    width:20em;
}

.clear {
    clear: both;
}

.acted {
    background-color:#00ff00;
}

.predictiontable {
    border: solid;
    border-collapse: collapse;
}

.predictiontable td {
    border: 1px solid;
    padding: 0.5em;
}
