<%inherit file="simple.html"/>

<%block name="title">Confirmation</%block>

<center>
  Action completed. <button onclick="goBack()">Go Back</button>
</center>

<script>
function goBack() {
    window.history.back();
}
</script>

