<!DOCTYPE html>
<html>
  <head>
    <meta name="viewport" content="width=device-width,user-scalable=yes"/>
    <script src="/static/js/checkuser.js"></script>
  </head>

  <body style="margin-left:5em; margin-top:2em;">

    <p>
      Please reset your password.
    </p>

    <form name="infoform" action="/resetpassword" method="POST">
      <input type="hidden" name="code" value="${code}">
      Password: <br>
      <input id="firstword" type="password" name="password"> <br>
      Confirm Password: <br>
      <input id="secondword" type="password" onkeyup="checkPassword()">
      <span id="confirmresult"></span> <br>
      <input type="submit" value="Submit">
    </form>

  </body>
</html>
