<%inherit file="simple.html"/>

<%block name="title">Request Password Reset</%block>

<p>
  Enter the email for the account you would like to recover.
</p>

<form name="infoform" method="POST">
  E-mail: <br>
  <input type="text" name="email"> <br>
  <input type="submit" value="Submit">
</form>
