<%inherit file="simple.html"/>

<%block name="title">Sent Reset Password</%block>

<h1>Email Sent</h1>
      
<!-- I should change returns based on this result -->
If ${email} was found in out records,
then a link should have been sent to allow you to reset your password.
