html {
    margin:1em auto;
    max-width:60em;
    background-color:#a0a0a0;
}

body {
    background-color:#ffffff;
    padding:2em;
}

h1, h2 {
    text-align:center;
}

h3 {
    background-color:#e6f7ff;
}

footer {
    padding:2em;
    background-color:#e6f7ff;
}

.acted {
    background-color:#00ff00;
}
