<%inherit file="simple.html"/>

<%block name="title">Action Submitted</%block>

<p><a href="../globalerror">Return To Global Errors</a></p>

<p>Submitted action to Unified with the following parameters:</p>

<h3>User</h3>
${user}

<h3>Workflows</h3>
% for wf in workflows:
${wf} <br>
% endfor

<h3>Action</h3>
${action}

<h3>Parameters</h3>
% for key, val in params.iteritems():
<b>${key}</b>: ${val} <br>
% endfor

<h3>Reasons</h3>
% for reason in reasons:
<h4>${reason['short']}</h4>
<p>${reason['long']}</p>
% endfor

<p><a href="../globalerror">Return To Global Errors</a></p>
