table {
    float: left;
    margin: 0px 25px 0px 25px;
}

tr:hover {
    background-color: #ddd;
}

.remover:hover {
    background-color: #999;
}

.ui-tooltip {
    max-width: 500px;
    font-family: monospace;
}

.dot {
    height: 15px;
    width: 15px;
    background-color: #bbb;
    border-radius: 50%;
    display: inline-block;
}
