<%inherit file="simple.html"/>

<%block name="title">Account Activated</%block>

<h1>Account Activated!</h1>

<p>
  The account for ${user} has been activated.
  Now actions you submit will be taken by Unified.
</p>
