<%inherit file="simple.html"/>

<%block name="title">No Action</%block>

You must choose an action to do that! <br>
% if workflow:
<a href="/seeworkflow?workflow=${workflow}">Go Back!</a> (and start over...)
% else:
<a href="/">Go Back!</a>
% endif
