<%inherit file="simple.html"/>

<%block name="title">Go Activate</%block>

<h1>Go Activate Your account! </h1>

<p>
  An email has been sent to ${email} with an activation link.
  After following that link,
  you will be able to log into your account to submit actions.
</p>
