<!DOCTYPE html>
<html>

  <head>
    <meta name="viewport" content="width=device-width,user-scalable=yes"/>
    <title><%block name="title"/></title>
    <link rel="stylesheet" type="text/css" href="/static/css/style.css"></link>
    <%block name="scripts"/>
  </head>

  <body>
    ${self.body()}

  <footer>
    <center>
      <a href="/">Return to the Home page</a> <br>
      <a href="/globalerror">Global Errors</a> <br>
      <a href="/resetcache">Manually reset user's session cache of global errors</a> <br>
      <a href="/cluster">Update server cluster definition</a>
    </center>
  </footer>
  </body>

</html>
